@@ -50,13 +50,6 @@
 #include "gvfsdaemonprotocol.h"
 #include "gvfskeyring.h"
 
-#ifdef HAVE_GCONF
-#include <gconf/gconf-client.h>
-#endif
-
-/* We load a default workgroup from gconf */
-#define PATH_GCONF_GNOME_VFS_SMB_WORKGROUP "/system/smb/workgroup"
-
 #include <libsmbclient.h>
 #include "libsmb-compat.h"
 
@@ -68,6 +61,8 @@ struct _GVfsBackendSmb
   char *share;
   char *user;
   char *domain;
+  char *path;
+  char *default_workgroup;
   
   SMBCCTX *smb_context;
 
@@ -91,7 +86,6 @@ struct _GVfsBackendSmb
   SMBCSRV *cached_server;
 };
 
-static char *default_workgroup = NULL;
 
 G_DEFINE_TYPE (GVfsBackendSmb, g_vfs_backend_smb, G_VFS_TYPE_BACKEND)
 
@@ -113,6 +107,8 @@ g_vfs_backend_smb_finalize (GObject *object)
   g_free (backend->server);
   g_free (backend->user);
   g_free (backend->domain);
+  g_free (backend->path);
+  g_free (backend->default_workgroup);
   
   if (G_OBJECT_CLASS (g_vfs_backend_smb_parent_class)->finalize)
     (*G_OBJECT_CLASS (g_vfs_backend_smb_parent_class)->finalize) (object);
@@ -121,6 +117,19 @@ g_vfs_backend_smb_finalize (GObject *object)
 static void
 g_vfs_backend_smb_init (GVfsBackendSmb *backend)
 {
+  char *workgroup;
+  GSettings *settings;
+
+  /* Get default workgroup name */
+  settings = g_settings_new ("org.gnome.system.smb");
+
+  workgroup = g_settings_get_string (settings, "workgroup");
+  if (workgroup && workgroup[0])
+    backend->default_workgroup = workgroup;
+  else
+    g_free (workgroup);
+
+  g_object_unref (settings);
 }
 
 /**
@@ -502,8 +511,8 @@ do_mount (GVfsBackend *backend,
   smbc_setFunctionPurgeCachedServers (smb_context, purge_cached);
 
   /* FIXME: is strdup() still needed here? -- removed */
-  if (default_workgroup != NULL)
-    smbc_setWorkgroup (smb_context, default_workgroup);
+  if (op_backend->default_workgroup != NULL)
+    smbc_setWorkgroup (smb_context, op_backend->default_workgroup);
 
 #ifndef DEPRECATED_SMBC_INTERFACE
   smb_context->flags = 0;
@@ -558,7 +567,11 @@ do_mount (GVfsBackend *backend,
   g_vfs_backend_set_mount_spec (backend, smb_mount_spec);
   g_mount_spec_unref (smb_mount_spec);
 
-  uri = create_smb_uri (op_backend->server, op_backend->share, NULL);
+  /* FIXME: we're stat()-ing user-specified path here, not the root. Ideally we
+            would like to fallback to root when first mount attempt fails, though
+            it would be tough to actually say if it was an authentication failure
+            or the particular path problem. */
+  uri = create_smb_uri (op_backend->server, op_backend->share, op_backend->path);
 
 
   /*  Samba mount loop  */
@@ -615,6 +628,7 @@ do_mount (GVfsBackend *backend,
 
   /* Mount was successful */
 
+  g_vfs_backend_set_default_location (backend, op_backend->path);
   g_vfs_keyring_save_password (op_backend->last_user,
 			       op_backend->server,
 			       op_backend->last_domain,
@@ -636,7 +650,7 @@ try_mount (GVfsBackend *backend,
 	   gboolean is_automount)
 {
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
-  const char *server, *share, *user, *domain;
+  const char *server, *share, *user, *domain, *path;
 
   server = g_mount_spec_get (mount_spec, "server");
   share = g_mount_spec_get (mount_spec, "share");
@@ -651,11 +665,13 @@ try_mount (GVfsBackend *backend,
 
   user = g_mount_spec_get (mount_spec, "user");
   domain = g_mount_spec_get (mount_spec, "domain");
+  path = mount_spec->mount_prefix;
   
   op_backend->server = g_strdup (server);
   op_backend->share = g_strdup (share);
   op_backend->user = g_strdup (user);
   op_backend->domain = g_strdup (domain);
+  op_backend->path = g_strdup (path);
   
   return FALSE;
 }
@@ -1615,13 +1631,17 @@ do_query_fs_info (GVfsBackend *backend,
 
       if (res == 0)
         {
-          /* FIXME: inconsistent return values (libsmbclient-3.4.2)
-           *       - for linux samba hosts, f_frsize is zero and f_bsize is a real block size
-           *       - for some Windows hosts (XP), f_frsize and f_bsize should be multiplied to get real block size
-           */
-          g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE, st.f_bsize * st.f_blocks * ((st.f_frsize == 0) ? 1 : st.f_frsize));
-          g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE, st.f_bsize * st.f_bfree * ((st.f_frsize == 0) ? 1 : st.f_frsize));
-          g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY, st.f_flag & SMBC_VFS_FEATURE_RDONLY);
+          /* older samba versions ( < 3.0.28) return zero values in struct statvfs */
+          if (st.f_blocks > 0)
+            {
+              /* FIXME: inconsistent return values (libsmbclient-3.4.2)
+               *       - for linux samba hosts, f_frsize is zero and f_bsize is a real block size
+               *       - for some Windows hosts (XP), f_frsize and f_bsize should be multiplied to get real block size
+               */
+              g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE, st.f_bsize * st.f_blocks * ((st.f_frsize == 0) ? 1 : st.f_frsize));
+              g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE, st.f_bsize * st.f_bfree * ((st.f_frsize == 0) ? 1 : st.f_frsize));
+              g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY, st.f_flag & SMBC_VFS_FEATURE_RDONLY);
+            }
         }
     }
 #endif
@@ -1704,6 +1724,15 @@ do_set_attribute (GVfsBackend *backend,
 
   if (strcmp (attribute, G_FILE_ATTRIBUTE_TIME_MODIFIED) == 0)
     {
+      if (type != G_FILE_ATTRIBUTE_TYPE_UINT64) 
+        {
+          g_vfs_job_failed (G_VFS_JOB (job),
+                            G_IO_ERROR,
+                            G_IO_ERROR_INVALID_ARGUMENT,
+                            "%s",
+                            _("Invalid attribute type (uint64 expected)"));
+        }
+
       smbc_utimes = smbc_getFunctionUtimes (op_backend->smb_context);
       tbuf[1].tv_sec = (*(guint64 *)value_p);  /* mtime */
       tbuf[1].tv_usec = 0;
@@ -2105,9 +2134,6 @@ g_vfs_backend_smb_class_init (GVfsBackendSmbClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GVfsBackendClass *backend_class = G_VFS_BACKEND_CLASS (klass);
-#ifdef HAVE_GCONF
-  GConfClient *gclient;
-#endif
   
   gobject_class->finalize = g_vfs_backend_smb_finalize;
 
@@ -2134,25 +2160,6 @@ g_vfs_backend_smb_class_init (GVfsBackendSmbClass *klass)
   backend_class->move = do_move;
   backend_class->try_query_settable_attributes = try_query_settable_attributes;
   backend_class->set_attribute = do_set_attribute;
-
-#ifdef HAVE_GCONF
-  gclient = gconf_client_get_default ();
-  if (gclient)
-    {
-      char *workgroup;
-      
-      workgroup = gconf_client_get_string (gclient, 
-					   PATH_GCONF_GNOME_VFS_SMB_WORKGROUP, NULL);
-
-      if (workgroup && workgroup[0])
-	default_workgroup = workgroup;
-      else
-	g_free (workgroup);
-      
-      g_object_unref (gclient);
-    }
-#endif
-
 }
 
 void
