@@ -40,7 +40,7 @@
 
 /* Originally from vte */
 
-#include "../config.h"
+#include "config.h"
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
@@ -348,7 +348,7 @@ _pty_run_on_pty(int fd, gboolean login,
 #ifdef HAVE_UTMP_H
 	/* This sets stdin, stdout, stderr to the socket */	
 	if (login && login_tty (fd) == -1) {
-		g_printerr ("mount child process login_tty failed: %s\n", strerror (errno));
+		g_printerr ("mount child process login_tty failed: %s\n", g_strerror (errno));
 		return -1;
 	}
 #endif
@@ -448,6 +448,7 @@ _pty_fork_on_pty_name(const char *path, int parent_fd, char **env_add,
 		break;
 	case 0:
 		/* Child. Close the parent's ends of the pipes. */
+		close(parent_fd);
 		close(ready_a[0]);
 		close(ready_b[1]);
 	
